<?php
/**
 * Perforce Swarm
 *
 * @copyright   2013-2020 Perforce Software. All rights reserved.
 * @license     Please see LICENSE.txt in top-level readme folder of this distribution.
 * @version     2019.3/2027974
 */

namespace Changes\Service;

use Application\Service\P4Command;
use P4\Connection\ConnectionInterface as Connection;
use P4\Spec\Change as P4Change;
use P4\Connection\CommandResult;

/**
 * Class Change
 * @package Changes\Service
 */
class Change extends P4Command
{
    const DESCRIBE_COMMAND = 'describe';
    const REOPEN_COMMAND   = 'reopen';
    const SHELVE_COMMAND   = 'shelve';
    const UNSHELVE_COMMAND = 'unshelve';
    const AFFECTED_FILES   = 'Affected files ...';
    const SHELVED_FILES    = 'Shelved files ...';
    const FILE_LINE_PREFIX = '#';
    const STREAM_FIELD     = 'stream';
    // Flags
    const FILES_ONLY         = '-Af';
    const PENDING_CHANGELIST = '-s';

    /**
     * @param Connection $connection
     * @param array $options
     * @param P4Change $change
     * @return CommandResult
     */
    public function describe(Connection $connection, array $options, P4Change $change)
    {
        return $this->run($connection, self::DESCRIBE_COMMAND, $options, [$change->getId()]);
    }

    /**
     * @param Connection $connection
     * @param array $options
     * @param P4Change $change
     * @return array
     */
    public function getFileList(Connection $connection, array $options, P4Change $change)
    {
        $inHeader     = true;
        $files        = [];
        $describeData = $this->describe($connection, $options, $change)->getData();
        foreach ($describeData as $data) {
            $data = trim($data, "\r\n");
            // if we are in the header check to see if we have hit the start
            // of the file list and return that we took care of this block.
            if ($inHeader) {
                if (!strlen($data) || $data === self::AFFECTED_FILES || $data === self::SHELVED_FILES) {
                    $inHeader = false;
                }
                continue;
            }
            $files[] = substr($data, 0, strrpos($data, self::FILE_LINE_PREFIX));
        }
        return $files;
    }
    /**
     * Get the stream for a given changelist if present.
     *
     * @param Connection $connection The P4 connection.
     * @param P4Change   $change     The changelist we want to get the spec for.
     * @return string | null
     */
    public function getStream(Connection $connection, P4Change $change)
    {
        // Must run describe to get the stream spec data as the changelist doesn't have the stream
        // spec against it when it is shelved.
        $describe = $this->describe($connection, [self::COMMAND_FLAGS => ['-sS']], $change)->getData(0);
        return isset($describe[self::STREAM_FIELD]) ? $describe[self::STREAM_FIELD] : null;
    }

    /**
     * Running the reopen. The path is apart of the options input then flags ar apart of the options flags.
     * @param Connection $connection
     * @param array $options
     * @param P4Change $change
     * @return CommandResult
     */
    public function reopen(Connection $connection, array $options, P4Change $change)
    {
        // Changelist first then the flags followed by location.
        // p4 reopen [-c changelist#] [-t filetype] [-Si] file ... (2019.1 help output)
        $options = [
            P4Command::COMMAND_FLAGS => array_merge(
                ['-c', $change->getId()],
                isset($options[P4Command::COMMAND_FLAGS])? $options[P4Command::COMMAND_FLAGS]: [],
                isset($options[P4Command::INPUT]) ? $options[P4Command::INPUT] :[]
            )
        ];
        return $this->run($connection, self::REOPEN_COMMAND, $options);
    }

    /**
     * Running the shelve. The path is apart of the options input then flags ar apart of the options flags.
     * @param Connection $connection
     * @param array $options
     * @param P4Change $change
     * @return CommandResult
     */
    public function shelve(Connection $connection, array $options, P4Change $change)
    {
        // flags come before changelist then location lasted.
        // p4 shelve [-Af|-As] [-a option] [-p] -c changelist# [-f] [file ...] (2019.1 help output)
        $options = [
            P4Command::COMMAND_FLAGS => array_merge(
                isset($options[P4Command::COMMAND_FLAGS])? $options[P4Command::COMMAND_FLAGS]: [],
                ['-c', $change->getId()],
                isset($options[P4Command::INPUT]) ? $options[P4Command::INPUT] :[]
            )
        ];
        return $this->run($connection, self::SHELVE_COMMAND, $options);
    }

    /**
     * Running the unshelve. The path is apart of the options input then flags ar apart of the options flags.
     * @param Connection $connection
     * @param array $options
     * @param P4Change $change
     * @return CommandResult
     */
    public function unshelve(Connection $connection, array $options, P4Change $change)
    {
        // flags come before changelist then location lasted.
        //  p4 unshelve -s changelist# [options] [file ...] (2019.1 help output)
        $options = [
            P4Command::COMMAND_FLAGS => array_merge(
                ['-s', $change->getId()],
                isset($options[P4Command::COMMAND_FLAGS])? $options[P4Command::COMMAND_FLAGS]: [],
                isset($options[P4Command::INPUT]) ? $options[P4Command::INPUT] :[]
            )
        ];
        return $this->run($connection, self::UNSHELVE_COMMAND, $options);
    }
}
