<?php
/**
 * Perforce Swarm
 *
 * @copyright   2013-2020 Perforce Software. All rights reserved.
 * @license     Please see LICENSE.txt in top-level readme folder of this distribution.
 * @version     2019.3/2027974
 */

use Application\Config\ConfigManager;
use Application\Controller\IndexControllerFactory;
use Application\Factory\InvokableServiceFactory;
use Application\View\Helper\ViewHelperFactory;
use Reviews\Controller\IndexController;
use Reviews\Filter\Keywords;
use Application\Config\Services;
use Reviews\Validator\Transitions;

return [
    'reviews' => [
        'patterns' => [
            'octothorpe' => [     // #review or #review-1234 with surrounding whitespace/eol
                                  'regex'  => '/(?P<pre>(?:\s|^)\(?)'
                    . '\#(?P<keyword>review|append|replace)(?:-(?P<id>[0-9]+))?'
                    . '(?P<post>[.,!?:;)]*(?=\s|$))/i',
                                  'spec'   => '%pre%#%keyword%-%id%%post%',
                                  'insert' => "%description%\n\n#review-%id%",
                                  // Ignore line limit here to aid readability
                                  // @codingStandardsIgnoreStart
                                  'strip'  => '/^\s*\#(review|append|replace)(-[0-9]+)?(\s+|$)|(\s+|^)\#(review|append|replace)(-[0-9]+)?\s*$/i'
                                  // @codingStandardsIgnoreEnd
            ],
            'leading-square'  => [ // [review] or [review-1234] at start
                'regex'  => '/^(?P<pre>\s*)\[(?P<keyword>review|append|replace)(?:-(?P<id>[0-9]+))?\](?P<post>\s*)/i',
                'spec'   => '%pre%[%keyword%-%id%]%post%'
            ],
            'trailing-square' => [ // [review] or [review-1234] at end
                'regex'  => '/(?P<pre>\s*)\[(?P<keyword>review|append|replace)(?:-(?P<id>[0-9]+))?\](?P<post>\s*)?$/i',
                'spec'   => '%pre%[%keyword%-%id%]%post%'
            ],
        ],
        'filters' => [
            'fetch-max' => 50,
            'filter-max' => 50,
            'result_sorting' => true,
            'date_field' => 'created', // 'created' displays and sorts by created date, 'updated' displays and sorts
            // by last updated
            // These need to match Review::FETCH_BY...
            'hasVoted' => [
                'fetch-max' => 5000,
            ],
            'myComments' => [
                'fetch-max' => 2500,
            ],
        ],
        'expand_group_reviewers' => false, // whether swarm should expand group members on the review page if they
        // have been added as an individual.
        'cleanup'              => [
            'mode'        => 'user', // auto - follow default, user - present checkbox(with default)
            'default'     => false,  // clean up pending changelists on commit
            'reopenFiles' => false   // re-open any opened files into the default changelist
        ],
        'disable_commit'        => false,
        'disable_self_approve'  => false, // whether authors can approve their own reviews
        'commit_credit_author'  => true,
        'commit_timeout'        => 1800,  // default: 30 minutes (must be in seconds)
        'unapprove_modified'    => true,  // whether approved reviews with modified files can be automatically
        // unapproved
        'ignored_users'         => [],
        'allow_author_change'   => false, // Whether anyone can change the Author
        'sync_descriptions'     => false, // if true a changesaved event will update all reviews attached to said change
        'expand_all_file_limit' => 10,    // Controls if 'Expand all' is available for reviews by specifying a file
        // limit over which the option will not be available. 0 signifies always on.
        'process_shelf_delete_when' => [], // States of the review we will process if a user deletes files from
        // their shelved changelist.
        // Supported states array('needsReview', 'needsRevision', 'archived', 'rejected', 'approved')
        'disable_tests_on_approve_commit' => false, // if set to true then automated tests will not be called when a
        // review is approved and committed
        'disable_approve_when_tasks_open' => false, // false shows a warning if tasks are open, true prevents approve
        'version_chooser'                 => 'chooser',
        'more_context_lines'              => 10,
        'max_bottom_context_lines'        => 10000,
        'allow_author_obliterate'         => false, // If true author will be allowed to obliterate their own review.
        'moderator_approval'              => ConfigManager::VALUE_ANY, // for reviews with multiple branches either
        // 'any' moderator may approve or 'each' must
        // approve (at least one from each branch)
        'end_states'                      => [],  // States that are considered closed
        // when checking workflow rules to
        // see if a review can be changed
    ],
    'security' => [
        'login_exempt'  => ['review-tests', 'review-deploy']
    ],
    'router' => [
        'routes' => [
            'review' => [
                'type' => 'Zend\Router\Http\Segment',
                'options' => [
                    'route'    => '/review[s]/:review[/v[:version]][/]',
                    'defaults' => [
                        'controller' => IndexController::class,
                        'action'     => 'review',
                        'version'    => null
                    ],
                ],
            ],
            'review-version-delete' => [
                'type' => 'Zend\Router\Http\Segment',
                'options' => [
                    'route'    => '/review[s]/:review/v:version/delete[/]',
                    'defaults' => [
                        'controller' => IndexController::class,
                        'action'     => 'deleteVersion',
                        'version'    => null
                    ],
                ],
            ],
            'review-reviewer' => [
                'type' => 'Zend\Router\Http\Segment',
                'options' => [
                    'route'    => '/review[s]/:review/reviewers/:user[/]',
                    'defaults' => [
                        'controller' => IndexController::class,
                        'action'     => 'reviewer',
                        'user'       => null
                    ],
                ],
            ],
            'review-reviewers' => [
                'type' => 'Zend\Router\Http\Segment',
                'options' => [
                    'route'    => '/review[s]/:review/reviewers[/]',
                    'defaults' => [
                        'controller' => IndexController::class,
                        'action'     => 'reviewers'
                    ],
                ],
            ],
            'review-author' => [
                'type' => 'Zend\Router\Http\Segment',
                'options' => [
                    'route'    => '/review[s]/:review/author[/:author][/]',
                    'defaults' => [
                        'controller' => IndexController::class,
                        'action'     => 'editAuthor',
                        'author'     => null
                    ],
                ],
            ],
            'review-vote' => [
                'type' => 'Zend\Router\Http\Segment',
                'options' => [
                    'route'    => '/review[s]/:review/vote/:vote[/]',
                    'defaults' => [
                        'controller' => IndexController::class,
                        'action'     => 'vote',
                        'vote'       => null
                    ],
                ],
            ],
            'review-tests' => [
                'type' => 'Zend\Router\Http\Segment',
                'options' => [
                    'route'    => '/review[s]/:review/tests/:status[/:token][/]',
                    'defaults' => [
                        'controller' => IndexController::class,
                        'action'     => 'testStatus',
                        'status'     => null,
                        'token'      => null
                    ],
                ],
            ],
            'review-deploy' => [
                'type' => 'Zend\Router\Http\Segment',
                'options' => [
                    'route'    => '/review[s]/:review/deploy/:status[/:token][/]',
                    'defaults' => [
                        'controller' => IndexController::class,
                        'action'     => 'deployStatus',
                        'status'     => null,
                        'token'      => null
                    ],
                ],
            ],
            'review-transition' => [
                'type' => 'Zend\Router\Http\Segment',
                'options' => [
                    'route'    => '/review[s]/:review/transition[/]',
                    'defaults' => [
                        'controller' => IndexController::class,
                        'action'     => 'transition'
                    ],
                ],
            ],
            'dashboards' => [
                'type' => 'Zend\Router\Http\Segment',
                'options' => [
                    'route' => '/dashboards/action',
                    'defaults' => [
                        'controller' => IndexController::class,
                        'action'     => 'dashboard',
                        'author' => null
                    ],
                ],
            ],
            'reviews' => [
                'type' => 'Zend\Router\Http\Segment',
                'options' => [
                    'route'    => '/review[s][/]',
                    'defaults' => [
                        'controller' => IndexController::class,
                        'action'     => 'index'
                    ],
                ],
            ],
            'add-review' => [
                'type' => 'Zend\Router\Http\Segment',
                'options' => [
                    'route'    => '/review[s]/add[/]',
                    'defaults' => [
                        'controller' => IndexController::class,
                        'action'     => 'add'
                    ],
                ],
            ],
            'review-file' => [
                'type' => 'Application\Router\Regex',
                'options' => [
                    'regex'    => '/reviews?/(?P<review>[0-9]+)/v(?P<version>[0-9,]+)/files?(/(?P<file>.*))?',
                    'spec'     => '/reviews/%review%/v%version%/files/%file%',
                    'defaults' => [
                        'controller' => IndexController::class,
                        'action'     => 'fileInfo',
                        'review'     => null,
                        'version'    => null,
                        'file'       => null
                    ],
                ],
            ],
        ],
    ],
    'controllers' => [
        'factories' => [
            IndexController::class => IndexControllerFactory::class,
        ],
    ],
    'service_manager' => [
        'aliases' => [
            Keywords::SERVICE     => Keywords::class,
            Services::TRANSITIONS => Transitions::class
        ],
        'factories' => array_fill_keys(
            [
                Keywords::class,
                Transitions::class,
            ],
            InvokableServiceFactory::class
        )
    ],
    'view_manager' => [
        'template_path_stack'   => [
            __DIR__ . '/../view',
        ],
    ],
    'view_helpers' => [
        'factories' => array_fill_keys(
            [
                ViewHelperFactory::REVIEWS,
                ViewHelperFactory::REVIEW_KEYWORDS,
                ViewHelperFactory::REVIEWERS_CHANGES,
                ViewHelperFactory::AUTHOR_CHANGE,
            ],
            ViewHelperFactory::class
        )
    ],
];
